% (c)2011, Thomas Galliker
% Please report updates/corrections to
% thomas_galliker@bluewin.ch
% 
% Thank you

-module(test2).
-export([start/2]).
-define(CHARSET, "abcdefghijklmnopqrstuvwxyz").
-define(NODESETUPTIME, 300). % avg time per node to setup the ring
-define(ERLANGNODE, 'node1@172.20.10.2'). 

start(NodeCount, WriteCount)->
    A = node3:start(0),
    register(nodestart, A),
    WaitForRing = (2+NodeCount) * ?NODESETUPTIME,
    TestProc = spawn(fun() -> run({nodestart, ?ERLANGNODE}, WriteCount, 0) end),
    setupRing(A, NodeCount-1, NodeCount, [TestProc], WriteCount, WaitForRing),
    A.

% Setup a ring of nodes on this local erlang node
setupRing(_, 0, _, TestProcList, _, WaitForRing) ->
    timer:sleep(WaitForRing),
    startTest(TestProcList),
    io:format("All tests started!~n", []);
setupRing(RefNode, NodeCount, MaxNodeCount, TestProcList, WriteCount, WaitForRing) ->
    % to get significant results, we have to make sure
    % we use evenly distributed nodes.
    NodeId = NodeCount * (WriteCount/MaxNodeCount), 
    %io:format("NEW NodeId=~w~n", [NodeId]),
    X = node3:start(NodeId,nodestart),
    Nodename = list_to_atom(lists:flatten(string:concat("node_",get_random_string(4)))),
    %Nodename = nodename,
    register(Nodename, X),
    %timer:sleep(10),
    TestProc = spawn(fun() -> run({Nodename, ?ERLANGNODE}, WriteCount, 0) end),
    io:format("Registering node ~w~n", [Nodename]),
    NewTestProcList = [TestProc|TestProcList],
    setupRing(RefNode, NodeCount-1, MaxNodeCount, NewTestProcList, WriteCount, WaitForRing).

%Signal test processes to start!
startTest([TestProc|TestProcList]) ->
    TestProc ! start,
    startTest(TestProcList);
startTest([]) ->
    true.

run(Node, Count, Offset) ->
    %io:format("Node ~w waiting for start signal...~n", [Node]),
    receive
	start ->
	    %io:format("Starting ~w WRITES to ~w~n", [Count, Node]),
	    Start = now(),
	    doWrite(Node, Count, Offset)
	%Else ->
	%    Start = nil,
	%    io:format("~w: Test got unknown message: ~w~n", [Node, Else])
    end,
    %io:format("Starting ~w READS from ~w~n", [Count, Node]),
    doRead(Node, Count, Offset),
    Finish = now(),
    T =  timer:now_diff(Finish, Start) / 1000,
    io:format("Time for ~w entries: ~wms~n", [Count, T]),
    run(Node, Count, Offset).

doWrite(_, 0, _) ->
    [];
doWrite(Node, Count, Offset) ->
    %io:format("SENDING {add, ~w, value, ref, self()} to ~w~n", [Count, Node]),
    Node ! {add, Count+Offset, value, ref, self()},
    doWrite(Node, Count-1, Offset).

doRead(_, 0, _) ->
	[];
doRead(Node, Count, Offset) ->
    Node ! {lookup, Count+Offset, Count, self()},
    receive
	{Count, Result, Key} -> []
	    %io:format("~w: Reading value ~w from node ~w~n", [Node, Result, Key])
    end,
    doRead(Node, Count-1, Offset).

get_random_string(Length) ->
    lists:foldl(fun(_, Acc) -> 
			[lists:nth(random:uniform(length(?CHARSET)), ?CHARSET)]
			    ++ Acc
		end, [], lists:seq(1, Length)).

